# -*- coding: utf-8 -*-
from datapurge.purge.policies import DummyPolicy, create_policy
from datapurge.compat import get_model

class PurgeTask(object):
    def __init__(self, model, policy=None, now=None):
        """
         :type policy: datapurge.purge.policies.BasePurgePolicy
        """
        self.model = model
        self.now = now
        self.policy = policy or DummyPolicy()

    def __call__(self, *args, **kwargs):
        self.policy.apply_policy(self.model, self.now)

    @classmethod
    def create_from_conf(cls, model_relation, options, force_now=None):
        app_label, model_name = model_relation.split(".")
        model = get_model(app_label, model_name)

        policy = create_policy(**options)
        return cls(model, policy, force_now)



# -*- coding: utf-8 -*-
from functools import partial
import django

if django.VERSION >= (1,4):
    from django.db.models import get_model
    get_model = partial(get_model, seed_cache=False, only_installed=True)
else:
    from django.db.models import get_model
    get_model = partial(get_model, seed_cache=False)


"""
This file demonstrates writing tests using the unittest module. These will pass
when you run "manage.py test".

Replace this with more appropriate tests for your application.
"""
import datetime

from django.test import TestCase
from django.contrib.auth.models import User
from datapurge.purge.actions import purge
from datapurge.purge.policies import CallablePolicy, ExpireFieldPolicy, LifetimePolicy
from datapurge.purge.tasks import PurgeTask


class TestWith100Users(TestCase):
    def setUp(self):
        dt = datetime.datetime.now()
        self.users = []

        # we use User model for test because it's in contrib and has datetime field we want
        for number in range(100):
            new_user = User.objects.create(
                username="testUser%s" % number,
                date_joined=dt - datetime.timedelta(days=number))
            self.users.append(new_user)

        #reverse to preserve chronological order
        self.users.reverse()

        self.assertEqual(User.objects.count(), 100)

class PoliciesTest(TestWith100Users):
    def test_CallablePolicy(self):
        expire_date = self.users[59].date_joined
        callable = lambda model: model.objects.filter(date_joined__lte=expire_date)

        policy = CallablePolicy(callable=callable)
        policy.apply_policy(User)

        self.assertEqual(User.objects.count(), 40)

    def test_ExpireFieldPolicy(self):
        fake_now = self.users[59].date_joined

        policy = ExpireFieldPolicy(expire_field="date_joined")
        policy.apply_policy(model=User, now=fake_now)

        self.assertEqual(User.objects.count(), 40)

    def test_LifetimePolicy(self):
        lifetime = self.users[-1].date_joined - self.users[59].date_joined
        fake_now = self.users[-1].date_joined

        policy = LifetimePolicy(lifetime=lifetime, created_field="date_joined")
        policy.apply_policy(model=User, now=fake_now)

        self.assertEqual(User.objects.count(), 40)

class PurgeTaskTest(TestWith100Users):
    def test_purge_by_expire_field(self):
        fake_now = self.users[59].date_joined

        relation = "auth.User"
        params =  {
            "expire_field": "date_joined",
        }

        task = PurgeTask.create_from_conf(relation, params, fake_now)
        task()

        self.assertEqual(User.objects.count(), 40)

    def test_purge_by_lifetime_field(self):
        lifetime = self.users[-1].date_joined - self.users[59].date_joined
        fake_now = self.users[-1].date_joined

        relation = "auth.User"
        params = {
            "lifetime": lifetime,
            "created_field": "date_joined"
        }

        task = PurgeTask.create_from_conf(relation, params, fake_now)
        task()

        self.assertEqual(User.objects.count(), 40)

    def test_purge_with_callable(self):
        expire_date = self.users[59].date_joined

        relation = "auth.User"
        params = {
            "callable": lambda model: model.objects.filter(date_joined__lte=expire_date)
        }

        task = PurgeTask.create_from_conf(relation, params)
        task()

        self.assertEqual(User.objects.count(), 40)

class PurgeActionTest(TestWith100Users):
    def test_purge_action(self):
        expire_date = self.users[59].date_joined

        # monkeypatch datapurge settings
        from datapurge import settings
        settings.DATAPURGE_MODELS = {
            "auth.User": {
                "callable": lambda model: model.objects.filter(date_joined__lte=expire_date)
            }
        }

        purge()

        self.assertEqual(User.objects.count(), 40)



# -*- coding: utf-8 -*-
import datetime
from django.conf import settings

DATAPURGE_GRACEFULLY = getattr(settings, "DATAPURGE_GRACEFULLY", False)
DATAPURGE_GRACE_STEP = getattr(settings, "DATAPURGE_GRACE_STEP", 100)
DATAPURGE_GRACE_WAIT = getattr(settings, "DATAPURGE_GRACE_WAIT", datetime.timedelta(seconds=0.3))

DATAPURGE_MODELS = getattr(settings, "DATAPURGE_MODELS", {})


# -*- coding: utf-8 -*-
class BaseDatapurgeException(BaseException):
    """Base datapurge app exception"""

class AmbiguousSettingsError(BaseException):
    """ Raises when datapurge settings appear to be ambiguous
    """


# -*- coding: utf-8 -*-
import datetime

from datapurge.exceptions import AmbiguousSettingsError
from datapurge.settings import DATAPURGE_GRACEFULLY

class BasePurgePolicy(object):
    """Base purge policy for purging old data"""

    def __init__(self, *args, **kwargs):
        self.grecefully = DATAPURGE_GRACEFULLY
        self.model = None

    def apply_policy(self, model, now=None):
        """ Applies purge policy to specified model.

        todo: creating self-purging (or not) logs
        :param model: Model class
        """
        # todo: timezone handling
        self.now = now or datetime.datetime.utcnow()
        self.model = model
        self.purge()

    def purge(self):
        """purges data gracefully or not

        todo: do it gracefully

        """
        if not self.grecefully:
            self.get_queryset().delete()
        else:
            raise NotImplemented

    def get_queryset(self):
        """ Gets model queryset
        :return: queryset to delete
        """
        raise NotImplemented


class CallablePolicy(BasePurgePolicy):

    def __init__(self, callable):
        super(CallablePolicy, self).__init__(callable)
        self.callable = callable

    def get_queryset(self):
        return self.callable(self.model)


class ExpireFieldPolicy(BasePurgePolicy):

    def __init__(self, expire_field):
        super(ExpireFieldPolicy, self).__init__(expire_field)
        self.expire_field = expire_field

    def get_queryset(self):
        query = {self.expire_field + '__lte': self.now}
        return self.model.objects.filter(**query)


class LifetimePolicy(BasePurgePolicy):

    def __init__(self, lifetime, created_field):
        super(LifetimePolicy, self).__init__(lifetime, created_field)
        self.lifetime = lifetime
        self.created_field = created_field

    def get_queryset(self):
        query = {self.created_field + '__lte': self.now - self.lifetime}
        return self.model.objects.filter(**query)


class DummyPolicy(BasePurgePolicy):
    def purge(self):
        pass


# Map settings kwargs to policy classes, use frozensets cause are hashable
POLICY_KWARGS_MAPPING = {
    frozenset(("callable",)): CallablePolicy,
    frozenset(("expire_field",)): ExpireFieldPolicy,
    frozenset(("lifetime", "created_field",)): LifetimePolicy,
    frozenset(): DummyPolicy,
}


def create_policy(**kwargs):
    """ Guess policy class by provided kwargs and create it's object
    """
    parameters = kwargs.keys()
    try:
        PolicyClass = POLICY_KWARGS_MAPPING.get(frozenset(parameters))
        return PolicyClass(**kwargs)
    except KeyError:
        raise AmbiguousSettingsError("Can not initialize any policy class with parameters %s" % kwargs)




# -*- coding: utf-8 -*-
import datetime

from datapurge import settings
from datapurge.purge.tasks import PurgeTask

def purge(now=None):
    # todo: timezone handling
    models = settings.DATAPURGE_MODELS
    now = now or datetime.datetime.utcnow()

    for model, conf in models.iteritems():
        task = PurgeTask.create_from_conf(model, conf, now)
        task()



#!/usr/bin/env python

# http://ericholscher.com/blog/2009/jun/29/enable-setuppy-test-your-django-apps/
# http://www.travisswicegood.com/2010/01/17/django-virtualenv-pip-and-fabric/
# http://code.djangoproject.com/svn/django/trunk/tests/runtests.py
import os
import sys

# fix sys path so we don't need to setup PYTHONPATH
sys.path.append(os.path.join(os.path.dirname(__file__), "../.."))
os.environ['DJANGO_SETTINGS_MODULE'] = 'datapurge.runtests.settings'

from django.conf import settings
from django.test.utils import get_runner


def main():

    TestRunner = get_runner(settings)

    test_runner = TestRunner(verbosity=2)
    failures = test_runner.run_tests(['datapurge'])

    sys.exit(failures)

if __name__ == '__main__':
    main()


# -*- coding: utf-8 -*-
import os
from setuptools import setup, find_packages

from datapurge import __version__ as version

README = open(os.path.join(os.path.dirname(__file__), 'README.md')).read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name = 'django-datapurge',
    version = version,
    packages = find_packages(),
    include_package_data = True,
    license = 'BSD License',
    description = 'A simple Django app to easily handle cleanup of old data (sessions, nonces, etc.)',
    long_description = README,
#    url = 'http://www.example.com/',
    author = 'Michał Jaworski',
    author_email = 'swistakm@gmail.com',
    classifiers = [
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
    test_suite='datapurge.runtests.runtests.main',
)




# -*- coding: utf-8 -*-
from django.core.management.base import BaseCommand, CommandError
from datapurge.exceptions import AmbiguousSettingsError
from datapurge.purge.actions import purge

class Command(BaseCommand):
    help = 'Purges old data'

    def handle(self, *args, **options):
        try:
            purge()
        except AmbiguousSettingsError, err:
            raise CommandError(str(err))
